#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Modified Stochastic Momentum Indicator
    /// </summary>
    [Description("Modified Stochastic Momentum Indicator")]
    public class StochasticMomentum : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int lookBack1 = 24; // Default setting for LookBack1
            private int lookBack2 = 100; // Default setting for LookBack2
            private int smooth1 = 3; // Default setting for Smooth1
            private int smooth2 = 10; // Default setting for Smooth2
        // User defined variables (add any user defined variables below)
			
			private DataSeries rdiff;
			private DataSeries diff;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Navy, 2), PlotStyle.Bar, "StoMom"));
			Plots[0].Pen.Width = 3;
            
			Add(new Line(Color.Gray, -40, "Lower"));
			Add(new Line(Color.Gray, 40, "Upper"));
			Add(new Line(Color.DarkGray, 0, "Zero line"));
			
			rdiff = new DataSeries(this);
			diff = new DataSeries(this);
			
			CalculateOnBarClose	= true;
			Overlay				= false;
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {	
			
			if (CurrentBar == 0)
			{
				StoMom.Set(0);
				return;
			}
			
			else
			{
			
			double HH1 = MAX( Typical, LookBack1 )[0];
			double LL1 = MIN( Typical, LookBack1 )[0];

			double HH2 = MAX( Typical, LookBack2 )[0];
			double LL2 = MIN( Typical, LookBack2 )[0];
			
			rdiff.Set(Close[0] - 0.5*(HH1 + LL1));
			diff.Set(HH2 - LL2);
			
			double avgrel = SMA(EMA(rdiff, Smooth1), Smooth2)[0];
			double avgdiff =SMA(EMA(diff, Smooth1), Smooth2)[0];
			
			
				
			//Calculating Stochastic Momentum
            
            StoMom.Set(100*avgrel/(0.5*avgdiff));
			}
			
			if (StoMom[0]<0 && StoMom[0]>StoMom[1])
			{
				PlotColors[0][0] = Color.Yellow;
			}		
			
			else if (StoMom[0]<0 && StoMom[0]<StoMom[1])
			{
				PlotColors[0][0] = Color.Red;
			}
			
			else if (StoMom[0]>0 && StoMom[0]>StoMom[1])
			{
				PlotColors[0][0] = Color.Green;
			}
			
			else if (StoMom[0]>0 && StoMom[0]<StoMom[1])
			{
				PlotColors[0][0] = Color.DarkBlue;
			}
			
			
			
			
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries StoMom
        {
            get { return Values[0]; }
        }
		
		

        [Description("LookBack1")]
        [GridCategory("Parameters")]
        public int LookBack1
        {
            get { return lookBack1; }
            set { lookBack1 = Math.Max(2, value); }
        }

        [Description("LookBack2")]
        [GridCategory("Parameters")]
        public int LookBack2
        {
            get { return lookBack2; }
            set { lookBack2 = Math.Max(2, value); }
        }

        [Description("Smooth1")]
        [GridCategory("Parameters")]
        public int Smooth1
        {
            get { return smooth1; }
            set { smooth1 = Math.Max(1, value); }
        }

        [Description("Smooth2")]
        [GridCategory("Parameters")]
        public int Smooth2
        {
            get { return smooth2; }
            set { smooth2 = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private StochasticMomentum[] cacheStochasticMomentum = null;

        private static StochasticMomentum checkStochasticMomentum = new StochasticMomentum();

        /// <summary>
        /// Modified Stochastic Momentum Indicator
        /// </summary>
        /// <returns></returns>
        public StochasticMomentum StochasticMomentum(int lookBack1, int lookBack2, int smooth1, int smooth2)
        {
            return StochasticMomentum(Input, lookBack1, lookBack2, smooth1, smooth2);
        }

        /// <summary>
        /// Modified Stochastic Momentum Indicator
        /// </summary>
        /// <returns></returns>
        public StochasticMomentum StochasticMomentum(Data.IDataSeries input, int lookBack1, int lookBack2, int smooth1, int smooth2)
        {
            if (cacheStochasticMomentum != null)
                for (int idx = 0; idx < cacheStochasticMomentum.Length; idx++)
                    if (cacheStochasticMomentum[idx].LookBack1 == lookBack1 && cacheStochasticMomentum[idx].LookBack2 == lookBack2 && cacheStochasticMomentum[idx].Smooth1 == smooth1 && cacheStochasticMomentum[idx].Smooth2 == smooth2 && cacheStochasticMomentum[idx].EqualsInput(input))
                        return cacheStochasticMomentum[idx];

            lock (checkStochasticMomentum)
            {
                checkStochasticMomentum.LookBack1 = lookBack1;
                lookBack1 = checkStochasticMomentum.LookBack1;
                checkStochasticMomentum.LookBack2 = lookBack2;
                lookBack2 = checkStochasticMomentum.LookBack2;
                checkStochasticMomentum.Smooth1 = smooth1;
                smooth1 = checkStochasticMomentum.Smooth1;
                checkStochasticMomentum.Smooth2 = smooth2;
                smooth2 = checkStochasticMomentum.Smooth2;

                if (cacheStochasticMomentum != null)
                    for (int idx = 0; idx < cacheStochasticMomentum.Length; idx++)
                        if (cacheStochasticMomentum[idx].LookBack1 == lookBack1 && cacheStochasticMomentum[idx].LookBack2 == lookBack2 && cacheStochasticMomentum[idx].Smooth1 == smooth1 && cacheStochasticMomentum[idx].Smooth2 == smooth2 && cacheStochasticMomentum[idx].EqualsInput(input))
                            return cacheStochasticMomentum[idx];

                StochasticMomentum indicator = new StochasticMomentum();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.LookBack1 = lookBack1;
                indicator.LookBack2 = lookBack2;
                indicator.Smooth1 = smooth1;
                indicator.Smooth2 = smooth2;
                Indicators.Add(indicator);
                indicator.SetUp();

                StochasticMomentum[] tmp = new StochasticMomentum[cacheStochasticMomentum == null ? 1 : cacheStochasticMomentum.Length + 1];
                if (cacheStochasticMomentum != null)
                    cacheStochasticMomentum.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheStochasticMomentum = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Modified Stochastic Momentum Indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.StochasticMomentum StochasticMomentum(int lookBack1, int lookBack2, int smooth1, int smooth2)
        {
            return _indicator.StochasticMomentum(Input, lookBack1, lookBack2, smooth1, smooth2);
        }

        /// <summary>
        /// Modified Stochastic Momentum Indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.StochasticMomentum StochasticMomentum(Data.IDataSeries input, int lookBack1, int lookBack2, int smooth1, int smooth2)
        {
            return _indicator.StochasticMomentum(input, lookBack1, lookBack2, smooth1, smooth2);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Modified Stochastic Momentum Indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.StochasticMomentum StochasticMomentum(int lookBack1, int lookBack2, int smooth1, int smooth2)
        {
            return _indicator.StochasticMomentum(Input, lookBack1, lookBack2, smooth1, smooth2);
        }

        /// <summary>
        /// Modified Stochastic Momentum Indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.StochasticMomentum StochasticMomentum(Data.IDataSeries input, int lookBack1, int lookBack2, int smooth1, int smooth2)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.StochasticMomentum(input, lookBack1, lookBack2, smooth1, smooth2);
        }
    }
}
#endregion
